## This file generated by InlineX::C2XS (version 0.15) using Inline::C (version 0.45_01)
package MyMod;
use strict;

require Exporter;
*import = \&Exporter::import;
require DynaLoader;

$MyMod::VERSION = '0.01';

DynaLoader::bootstrap MyMod $MyMod::VERSION;

@MyMod::EXPORT = ();
@MyMod::EXPORT_OK = qw(
    erf
    );

%MyMod::EXPORT_TAGS = (all => [qw(
    erf
    )]);

sub dl_load_flags {0} # Prevent DynaLoader from complaining and croaking

1;
